// Copyright 2010
// Author: Li Zhang
#include "./NussinovAlgorithm.h"
#include <stdio.h>
#include <sys/time.h>
#include <stdint.h>
#include <iostream>
#include <string>
#include <vector>
#include <utility>
using std::cout;
using std::endl;


// _____________________________________________________________________________
bool NussinovAlgorithm::isPaired(char xi, char xj)
{
  if ((xi == 'g' && xj == 'c') || (xi == 'c' && xj == 'g')) return true;
  if ((xi == 'a' && xj == 'u') || (xi == 'u' && xj == 'a')) return true;
  return false;
}

// _____________________________________________________________________________
void NussinovAlgorithm::initNussinov(const std::string& input)
{
  // Initialize the instance variables
  _sequence   = input;
  _maxNumPairs.resize(_sequence.length());
  for (size_t i = 0; i < _sequence.length(); ++i)
  {
    _maxNumPairs[i].resize(_sequence.length());
  }
  _traceBack.resize(_sequence.length());
  for (size_t i = 0; i < _sequence.length(); ++i)
  {
    _traceBack[i].resize(_sequence.length());
  }
}

// Find the maximum number of base pairs within a input sequence.
// _____________________________________________________________________________
void NussinovAlgorithm::findMaxPairs()
{
  int max, temp;
  for (int i = _sequence.length() - 1; i >= 0; --i)
  {
    for (int j = 0; j < static_cast<int>(_sequence.length()); ++j)
    {
      max = 0;
      //--------------------------------------------------------------
      // The algorithm looks at each recursive term to see whether it
      // gives the max num of pair. If the max. pair occurs in a
      // particular term, the max. num will be stored in a matrix with
      // the respective position indexed by i,j, and k. While doing that,
      // the algorithm also places a mark, such as -3, -2, -1, and k,
      // on the repective entry in a traceback matrix.
      //--------------------------------------------------------------
      // If i >= j - 4, then B[i,j] = 0. Since the matrix B is initialized
      // in the constructor, nothing needs to be done.
      if (i >= j)
      {
        _maxNumPairs[i][j] = 0;
        _traceBack[i][j] = -3;
      }
      else
      { // If i < j - 4, then it's valid for folding.
        // Case1: i,j paired together or none of i and j is paired
        temp = _maxNumPairs[i + 1][j - 1] +
               (isPaired(_sequence.at(i), _sequence.at(j)) ? 1 : 0);
        if (temp > max)
        {
          max = temp;
          _traceBack[i][j] = -1 -
             (isPaired(_sequence.at(i), _sequence.at(j)) ? 1 : 0);
        }
        // Case2: i, j paired with k such that 1 <= k < j
        for (int k = i; k < j; ++k)
        {
          temp = _maxNumPairs[i][k] + _maxNumPairs[k + 1][j];
          if (temp > max)
          {
            max = temp;
            _traceBack[i][j] = k;
          }
        }
        _maxNumPairs[i][j] = max;
      }
    }
  }
}


// Traceback the matrix.
// _____________________________________________________________________________
void NussinovAlgorithm::traceBack(int i, int j,
std::vector<std::pair<int, int> >* result)
{
  if (_traceBack[i][j] == -3) return;
  else if (_traceBack[i][j] == -2)
  {  // i and j paired together
    // alternativly result->operator[cnt] = make_pair(i,j);
    (*result).push_back(std::make_pair(i, j));
    traceBack(i + 1, j - 1, result);
  }
  else if (_traceBack[i][j] == -1)
  {
    traceBack(i + 1, j - 1, result);
  }
  else
  {  // i and j could be possibly paired with k such that i <= k < j
    int k = _traceBack[i][j];
    if ((i <= k) && ((k + 1) <= j))
    {
      traceBack(i, k, result);
      traceBack(k + 1, j, result);
    }
  }
}


// _____________________________________________________________________________
void NussinovAlgorithm::computeStructure(const std::string& input,
std::vector<std::pair<int, int> >* result)
{
  initNussinov(input);
  findMaxPairs();
  traceBack(0, _sequence.length() - 1, result);
}

// _____________________________________________________________________________
void NussinovAlgorithm::printMatrix() const
{
  cout <<"This is matrix _maxNumPairs:" << endl;
  for (size_t i = 0; i < _sequence.length(); ++i)
  {
    for (size_t j = 0; j < _sequence.length(); ++j)
    {
      if (_maxNumPairs[i][j] > 9)
      {
        cout << " " << _maxNumPairs[i][j];
      }
      else
      {
        cout << "  " << _maxNumPairs[i][j];
      }
    }
    cout << endl;
  }
  cout << "This is matrix _traceBack:" << endl;
  for (size_t i = 0; i < _sequence.length(); ++i)
  {
    for (size_t j = 0; j < _sequence.length(); ++j)
    {
      cout << " " << _traceBack[i][j];
    }
    cout << endl;
  }
}

// _____________________________________________________________________________
void NussinovAlgorithm::printStructure(const std::vector<std::pair<int, int> >&
    result) const
{
  for (size_t i = 0; i < result.size(); ++i)
  {
    printf("(%u, %u),", result[i].first + 1, result[i].second + 1);
    fflush(stdout);
  }
  printf("\n");
}
