// Copyright 2010
// Author: Li Zhang
#ifndef NUSSINOVALGORITHM_H_
#define NUSSINOVALGORITHM_H_

#include <gtest/gtest.h>
#include <string>
#include <vector>
#include <utility>
using std::string;
using std::vector;

class NussinovAlgorithm
{
 public:
  // Compute structure for the given input sequence (a string over the
  // alphabet {a,c,g,u}). Output is given as sequence of base pairs, where
  // each pair is a pair of indices in the string.
  FRIEND_TEST(NussinovAlgorithmTest, computeStructure);
  void computeStructure(const std::string& input,
      std::vector<std::pair<int, int> >* result);

  // Print the _maxNumPairs matrix (see below for an explanation of that
  // matrix).
  void printMatrix() const;
  FRIEND_TEST(NussinovAlgorithmTest, printMatrix);

  // Print the given structure.
  FRIEND_TEST(NussinovAlgorithmTest, printStructure);
  void printStructure(const std::vector<std::pair<int, int> >&
        result) const;

 private:
  // The input sequence for which we want to compute the maximal number of
  // base pairs. TODO(bast): rename to _sequence.
  string _sequence;

  // The dynamic programming matrix for computing the maximal number of base
  // pairs.
  vector< vector<int> > _maxNumPairs;

  // An auxiliary matrix for the traceback, to compute the structure belonging
  // to an entry in the _maxNumPairs matrix.
  vector< vector<int> > _traceBack;

  // Initialize the two matrices used by the Nussinov Algorithm (one for
  // calculating the maximum number of base pairs via dynamic programming, the
  // other for the trace back).
  void initNussinov(const std::string& input);
  FRIEND_TEST(NussinovAlgorithmTest, initNussinov);

  // Compute the maximal number of base pairs using dynamic programming.
  void findMaxPairs();
  FRIEND_TEST(NussinovAlgorithmTest, findMaxPairs);

  // Compute the structure for an entry in the _maxNumPairs matrix using a
  // trace back in that matrix.
  void traceBack(int i, int j,
                 std::vector<std::pair<int, int> >* result);
  FRIEND_TEST(NussinovAlgorithmTest, traceBack);

  // Return true if the given base pair matches (A-U or U-A or C-G or G-C) and
  // false otherwise.
  bool isPaired(char xi, char xj);
  FRIEND_TEST(NussinovAlgorithmTest, isPaired);
};

#endif  // NUSSINOVALGORITHM_H_
