// Copyright 2010
// Author: Li Zhang
#include <gtest/gtest.h>
#include <utility>
#include <vector>
#include <string>
#include "./NussinovAlgorithm.h"

// Test for the method computeStructure which includes
// initNussinov, findMaxPairs, traceback methods.
TEST(NussinovAlgorithmTest, computeStructure)
{
  NussinovAlgorithm nussinov;
  {
    // Test string "gauc".
    std::string input = "gauc";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 2);
    ASSERT_EQ(result[0].first, 0);
    ASSERT_EQ(result[0].second, 3);
    ASSERT_EQ(result[1].first, 1);
    ASSERT_EQ(result[1].second, 2);
  }

  // Test string "aucg".
  {
    std::string input = "aucg";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 2);
    ASSERT_EQ(result[0].first, 0);
    ASSERT_EQ(result[0].second, 1);
    ASSERT_EQ(result[1].first, 2);
    ASSERT_EQ(result[1].second, 3);
  }

  // Test string "acu".
  {
    std::string input = "acu";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 1);
    ASSERT_EQ(result[0].first, 0);
    ASSERT_EQ(result[0].second, 2);
  }

  // Test an empty string.
  {
    std::string input = "";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 0);
  }

  // Test string "acccccccu".
  {
    std::string input = "accccccuu";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 1);
    ASSERT_EQ(result[0].first, 0);
    ASSERT_EQ(result[0].second, 8);
  }

  // Test the string "gaaacuccca".
  {
    std::string input = "gaaacuccca";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 2);
    ASSERT_EQ(result[0].first, 0);
    ASSERT_EQ(result[0].second, 4);
    ASSERT_EQ(result[1].first, 5);
    ASSERT_EQ(result[1].second, 9);
  }

  // Test the string "acccucaaag".
  {
    std::string input = "acccucaaag";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 2);
    ASSERT_EQ(result[0].first, 1);
    ASSERT_EQ(result[0].second, 9);
    ASSERT_EQ(result[1].first, 4);
    ASSERT_EQ(result[1].second, 6);
  }

  // Test the string "acuccacu".
  {
    std::string input = "acuccacu";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 2);
    ASSERT_EQ(result[0].first, 0);
    ASSERT_EQ(result[0].second, 7);
    ASSERT_EQ(result[1].first, 2);
    ASSERT_EQ(result[1].second, 5);
  }

  // Test the string "acug".
  {
    std::string input = "acug";
    std::vector<std::pair<int, int> > result;
    nussinov.computeStructure(input, &result);
    ASSERT_EQ(result.size(), 1);
    ASSERT_EQ(result[0].first, 1);
    ASSERT_EQ(result[0].second, 3);
  }
}

// Test the isPaired method.
TEST(NussinovAlgorithmTest, isPaired)
{
  NussinovAlgorithm nussinov;
  {
    std::vector<bool> result;
    result.push_back(nussinov.isPaired('a', 'u'));
    result.push_back(nussinov.isPaired('c', 'g'));
    result.push_back(nussinov.isPaired('u', 'a'));
    result.push_back(nussinov.isPaired('a', 'u'));
    result.push_back(nussinov.isPaired('a', 'c'));
    result.push_back(nussinov.isPaired('a', 'g'));
    ASSERT_TRUE(result[0]);
    ASSERT_TRUE(result[1]);
    ASSERT_TRUE(result[2]);
    ASSERT_TRUE(result[3]);
    ASSERT_FALSE(result[4]);
    ASSERT_FALSE(result[5]);
  }
}
// Test the method initNussinov.
TEST(NussinovAlgorithmTest, initNussinov)
{
  NussinovAlgorithm nussinov;
  // Test the string "aaa".
  nussinov.initNussinov("aaa");
  ASSERT_EQ(nussinov._sequence, "aaa");
  ASSERT_EQ(nussinov._maxNumPairs.size(), 3);
  ASSERT_EQ(nussinov._maxNumPairs[0].size(), 3);
  ASSERT_EQ(nussinov._maxNumPairs[1].size(), 3);
  ASSERT_EQ(nussinov._maxNumPairs[2].size(), 3);
  ASSERT_EQ(nussinov._traceBack.size(), 3);
  ASSERT_EQ(nussinov._traceBack[0].size(), 3);
  ASSERT_EQ(nussinov._traceBack[1].size(), 3);
  ASSERT_EQ(nussinov._traceBack[2].size(), 3);
  // Test the empty string.
  nussinov.initNussinov("");
  ASSERT_EQ(nussinov._sequence, "");
  ASSERT_EQ(nussinov._maxNumPairs.size(), 0);
  ASSERT_EQ(nussinov._traceBack.size(), 0);
}

// Test the method findMaxPairs.
TEST(NussinovAlgorithmTest, findMaxPairs)
{
  NussinovAlgorithm nussinov;
  // Test the string "aaa".
  nussinov.initNussinov("aaa");
  nussinov.findMaxPairs();
  ASSERT_EQ(0, nussinov._maxNumPairs[0][2]);
  // Test the string "aucg".
  nussinov.initNussinov("aucg");
  nussinov.findMaxPairs();
  ASSERT_EQ(2, nussinov._maxNumPairs[0][3]);
  // Test the string "gauc".
  nussinov.initNussinov("gauc");
  nussinov.findMaxPairs();
  ASSERT_EQ(2, nussinov._maxNumPairs[0][3]);
  // Test the string "".
  nussinov.initNussinov("a");
  nussinov.findMaxPairs();
  ASSERT_EQ(0, nussinov._maxNumPairs[0][0]);
  // Test the string "acu".
  nussinov.initNussinov("acu");
  nussinov.findMaxPairs();
  ASSERT_EQ(1, nussinov._maxNumPairs[0][2]);
}

// Test the method traceBack
TEST(NussinovAlgorithmTest, traceBack)
{
  NussinovAlgorithm nussinov;
  vector< vector<int> > _traceBack;
  // Test for the matrix {{0, -2},{0, 0}}.
  nussinov._traceBack.resize(2);
  for (int i = 0; i < 2; i++)
  {
    nussinov._traceBack[i].resize(2);
  }
  nussinov._traceBack[0][0] = 0;
  nussinov._traceBack[0][1] = -2;
  nussinov._traceBack[1][0] = 0;
  nussinov._traceBack[1][1] = 0;
  std::vector<std::pair<int, int> > result;
  nussinov.traceBack(0, 1, &result);
  ASSERT_EQ(result.size(), 1);
  ASSERT_EQ(result[0].first, 0);
  ASSERT_EQ(result[0].second, 1);
  // Test the matrix, which the entries of it are all 0.
  nussinov._traceBack.resize(3);
  for (int i = 0; i < 3; i++)
  {
    nussinov._traceBack[i].resize(3);
  }
  nussinov._traceBack[0][0] = 0;
  nussinov._traceBack[0][1] = 0;
  nussinov._traceBack[0][2] = 0;
  nussinov._traceBack[1][0] = 0;
  nussinov._traceBack[1][1] = 0;
  nussinov._traceBack[1][2] = 0;
  nussinov._traceBack[2][0] = 0;
  nussinov._traceBack[2][1] = 0;
  nussinov._traceBack[2][2] = 0;
  std::vector<std::pair<int, int> > result1;
  nussinov.traceBack(0, 2, &result1);
  ASSERT_EQ(result1.size(), 0);
  // Test the matrix in which there are two entries are -2.
  nussinov._traceBack.resize(4);
  for (int i = 0; i < 4; i++)
  {
    nussinov._traceBack[i].resize(4);
  }
  nussinov._traceBack[0][0] = 0;
  nussinov._traceBack[0][1] = -2;
  nussinov._traceBack[0][2] = 0;
  nussinov._traceBack[0][3] = 1;
  nussinov._traceBack[1][0] = -2;
  nussinov._traceBack[1][1] = 2;
  nussinov._traceBack[1][2] = 0;
  nussinov._traceBack[1][3] = 2;
  nussinov._traceBack[2][0] = 3;
  nussinov._traceBack[2][1] = 1;
  nussinov._traceBack[2][2] = 0;
  nussinov._traceBack[2][3] = 1;
  nussinov._traceBack[3][0] = 0;
  nussinov._traceBack[3][1] = 1;
  nussinov._traceBack[3][2] = 0;
  nussinov._traceBack[3][3] = 3;
  std::vector<std::pair<int, int> > result2;
  nussinov.traceBack(0, 3, &result2);
  ASSERT_EQ(result2.size(), 2);
  ASSERT_EQ(result2[0].first, 0);
  ASSERT_EQ(result2[0].second, 1);
  ASSERT_EQ(result2[1].first, 1);
  ASSERT_EQ(result2[1].second, 0);
}





